@@ -1,6 +1,7 @@
 import { css } from '@emotion/css';
 
 import { GrafanaTheme2 } from '@grafana/data';
+import { Trans, t } from '@grafana/i18n';
 import { useStyles2, LinkButton } from '@grafana/ui';
 import { UpgradeBox } from 'app/core/components/Upgrade/UpgradeBox';
 
@@ -13,15 +14,22 @@ export function HighlightTrialReport() {
         featureId={'reporting-tab'}
         eventVariant={'trial'}
         featureName={'reporting'}
-        text={'Create unlimited reports during your trial of Grafana Pro.'}
+        text={t(
+          'reports.highlight-trial-report.text-create-unlimited-reports-during-trial-grafana',
+          'Create unlimited reports during your trial of Grafana Pro.'
+        )}
       />
-      <h3>Get started with reporting</h3>
+      <h3>
+        <Trans i18nKey="reports.highlight-trial-report.get-started-with-reporting">Get started with reporting</Trans>
+      </h3>
       <h6>
-        Reporting allows you to automatically generate PDFs from any of your dashboards and have Grafana email them to
-        interested parties on a schedule.
+        <Trans i18nKey="reports.highlight-trial-report.get-started-with-reporting-text">
+          Reporting allows you to automatically generate PDFs from any of your dashboards and have Grafana email them to
+          interested parties on a schedule.
+        </Trans>
       </h6>
       <LinkButton fill="text" href={'https://grafana.com/docs/grafana/latest/enterprise/reporting'}>
-        Learn more
+        <Trans i18nKey="reports.highlight-trial-report.learn-more">Learn more</Trans>
       </LinkButton>
     </div>
   );
